@@ -26,10 +26,11 @@
 
 
 enum {
-	VALIDATE,
+	VALIDATE_USER = 1,
+	VALIDATE_GROUP,
 	CANCEL,
-	SELECT_USER,
-	SELECT_GROUP,
+	//SELECT_USER,
+	//SELECT_GROUP,
 	LAST_SIGNAL
 };
 
@@ -38,8 +39,8 @@ static gint gtk_usergroup_signals[LAST_SIGNAL] = { 0 };
 static void gtk_usergroup_class_init (GtkUserGroupClass *class);
 static void gtk_usergroup_init (GtkUserGroup *gtk_usergroup);
 
-static void gtk_usergroup_on_user_clicked(GtkCellRenderer *cell, gchar *path_str, 
-	GtkListStore *list_users);
+/*static void gtk_usergroup_on_user_clicked(GtkCellRenderer *cell, gchar *path_str, 
+	GtkListStore *list_users);*/
 static void gtk_usergroup_on_validate(GtkWidget *button, gpointer *data);
 static void gtk_usergroup_on_cancel(GtkWidget *button, gpointer *data);
 static void gtk_usergroup_on_select_user(GtkTreeView *tree_view, gpointer user_data);
@@ -47,9 +48,9 @@ static void gtk_usergroup_on_select_group(GtkTreeView *tree_view, gpointer user_
 
 
 
-static GtkWidget *_create_list_view(GtkListStore *list, gchar *text_header);
-static void _fill_users_list(GtkListStore *list_users);
-static void _fill_groups_list(GtkListStore *list_groups);
+static void init_list_view(GtkTreeView *view, gchar *text_header);
+static void fill_users_list(GtkListStore *list_users);
+static void fill_groups_list(GtkListStore *list_groups);
 
 
 
@@ -75,7 +76,7 @@ GtkType gtk_usergroup_get_type(void)
 			(GInstanceInitFunc) gtk_usergroup_init
 		};
 
-		gtk_usergroup_type = g_type_register_static (GTK_TYPE_DIALOG, 
+		gtk_usergroup_type = g_type_register_static (GTK_TYPE_WINDOW, 
 				"GtkUserGroup", &gtk_usergroup_info, 0);
 	}
 
@@ -90,22 +91,77 @@ static void gtk_usergroup_class_init (GtkUserGroupClass *class)
 
 	object_class = (GtkObjectClass *) class;
   
-	/*gtk_usergroup_signals[VALIDATE] = 
-		g_signal_new ("validate",
+	gtk_usergroup_signals[VALIDATE_USER] = 
+		g_signal_new ("validate-user",
 				G_OBJECT_CLASS_TYPE(object_class),
-				GTK_RUN_FIRST,
-				G_STRUCT_OFFSET (GtkUserGroupClass, validate),
+				G_SIGNAL_RUN_FIRST,
+				G_STRUCT_OFFSET (GtkUserGroupClass, validate_user),
 				NULL, NULL,
-				g_cclosure_marshal_VOID__VOID, GTK_TYPE_NONE, 0);*/
+				g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
+	
+	gtk_usergroup_signals[VALIDATE_GROUP] = 
+		g_signal_new ("validate-group",
+				G_OBJECT_CLASS_TYPE(object_class),
+				G_SIGNAL_RUN_FIRST,
+				G_STRUCT_OFFSET (GtkUserGroupClass, validate_group),
+				NULL, NULL,
+				g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
   
-	class->validate = NULL;
+	class->validate_user  = NULL;
+	class->validate_group = NULL;
 }
 
 
 //
 static void gtk_usergroup_init (GtkUserGroup *gtk_usergroup)
 {
-	GtkDialog *dialog;
+	GladeXML  *xml;
+
+	xml = glade_xml_new(PACKAGE_DATA_DIR"/gtkusergroup.glade", "box_main", NULL);
+	if (!xml)
+	{
+		g_warning(_("Cannot load glade interface file"));
+		return;
+	}
+	gtk_usergroup->xml = xml;
+	
+	//
+	gtk_container_add(GTK_CONTAINER(gtk_usergroup), 
+		GTK_WIDGET(glade_xml_get_widget(gtk_usergroup->xml, "box_main")));
+	
+	//Users
+	gtk_usergroup->store_users = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
+	gtk_usergroup->view_users = GTK_TREE_VIEW(glade_xml_get_widget(gtk_usergroup->xml, "treeview_users"));
+	init_list_view(gtk_usergroup->view_users, _("Users"));
+	fill_users_list(gtk_usergroup->store_users);
+	gtk_tree_view_set_model(GTK_TREE_VIEW(gtk_usergroup->view_users ),
+		GTK_TREE_MODEL(gtk_usergroup->store_users) );
+	g_signal_connect(G_OBJECT(gtk_usergroup->view_users), "cursor-changed", 
+		G_CALLBACK(gtk_usergroup_on_select_user), gtk_usergroup);
+	
+	//Groups
+	gtk_usergroup->store_groups = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
+	gtk_usergroup->view_groups  = GTK_TREE_VIEW(glade_xml_get_widget(gtk_usergroup->xml, "treeview_groups"));
+	init_list_view(GTK_TREE_VIEW(gtk_usergroup->view_groups), _("Groups"));
+	fill_groups_list(gtk_usergroup->store_groups);
+	gtk_tree_view_set_model(GTK_TREE_VIEW(gtk_usergroup->view_groups),
+		GTK_TREE_MODEL(gtk_usergroup->store_groups) );
+	g_signal_connect(G_OBJECT(gtk_usergroup->view_groups), "cursor-changed", 
+		G_CALLBACK(gtk_usergroup_on_select_group), gtk_usergroup);
+	
+	//Button
+	g_signal_connect(G_OBJECT(glade_xml_get_widget(gtk_usergroup->xml, "button_validate")),
+		"clicked", G_CALLBACK(gtk_usergroup_on_validate), gtk_usergroup);
+	g_signal_connect(G_OBJECT(glade_xml_get_widget(gtk_usergroup->xml, "button_cancel")),
+		"clicked", G_CALLBACK(gtk_usergroup_on_cancel), gtk_usergroup);
+	
+	//Signals treeview
+	
+	
+	
+	return;
+	
+	/*GtkDialog *dialog;
 	GtkWidget *hbox_top, *hbox_bottom, *vhbox;
 	GtkWidget *label;
 	GtkWidget *button;
@@ -113,25 +169,25 @@ static void gtk_usergroup_init (GtkUserGroup *gtk_usergroup)
 	GtkWidget *scrolled;
 	
 	GtkWidget    *view_users, *view_groups;
-	GtkListStore *list_users, *list_groups;
+	GtkListStore *list_users, *list_groups;*/
 
 	
 	//DIALOG
-	dialog = GTK_DIALOG(gtk_usergroup);
+	//Dialog = GTK_DIALOG(gtk_usergroup);
 	
-	hbox_top = gtk_hbox_new(TRUE, 2);
+	/*hbox_top = gtk_hbox_new(TRUE, 2);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), GTK_WIDGET(hbox_top), TRUE, TRUE, 0);
-	gtk_widget_show(hbox_top);
+	gtk_widget_show(hbox_top);*/
 	
-	hbox_bottom = gtk_hbox_new(TRUE, 2);
+	/*hbox_bottom = gtk_hbox_new(TRUE, 2);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), GTK_WIDGET(hbox_bottom), TRUE, TRUE, 0);
-	gtk_widget_show(hbox_bottom);
+	gtk_widget_show(hbox_bottom);*/
 	
 	
 	//User
-	list_users = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
-	_fill_users_list(list_users);
-	view_users = _create_list_view(list_users, _("Users"));
+	//list_users = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
+	//_fill_users_list(list_users);
+	/*view_users = _create_list_view(list_users, _("Users"));
 	g_signal_connect(G_OBJECT(view_users), "cursor-changed", 
 		G_CALLBACK(gtk_usergroup_on_select_user), gtk_usergroup);
 	gtk_usergroup->list_users = list_users;
@@ -142,11 +198,11 @@ static void gtk_usergroup_init (GtkUserGroup *gtk_usergroup)
                                   	GTK_POLICY_AUTOMATIC);
 	gtk_container_add(GTK_CONTAINER(scrolled), view_users);
 	gtk_box_pack_start(GTK_BOX(hbox_top), GTK_WIDGET(scrolled), TRUE, TRUE, 0);
-	gtk_widget_show_all(scrolled);
+	gtk_widget_show_all(scrolled);*/
 	
 	
 	//Group
-	list_groups = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
+	/*list_groups = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
 	_fill_groups_list(list_groups);
 	view_groups = _create_list_view(list_groups, _("Groups"));
 	g_signal_connect(G_OBJECT(view_groups), "cursor-changed", 
@@ -158,11 +214,11 @@ static void gtk_usergroup_init (GtkUserGroup *gtk_usergroup)
                                   	GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_container_add(GTK_CONTAINER(scrolled), view_groups);
 	gtk_box_pack_start(GTK_BOX(hbox_top), GTK_WIDGET(scrolled), TRUE, TRUE, 0);
-	gtk_widget_show_all(scrolled);
+	gtk_widget_show_all(scrolled);*/
 
 
 	//Buttons
-	button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+	/*button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
 	gtk_box_pack_start(GTK_BOX(hbox_bottom), GTK_WIDGET(button), TRUE, TRUE, 0);
 	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(gtk_usergroup_on_cancel), gtk_usergroup);
 	gtk_widget_show(button);
@@ -170,7 +226,7 @@ static void gtk_usergroup_init (GtkUserGroup *gtk_usergroup)
 	button = gtk_button_new_from_stock(GTK_STOCK_OK);
 	gtk_box_pack_start(GTK_BOX(hbox_bottom), GTK_WIDGET(button), TRUE, TRUE, 0);
 	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(gtk_usergroup_on_validate), gtk_usergroup);
-	gtk_widget_show(button);
+	gtk_widget_show(button);*/
 }
 
 
@@ -184,40 +240,65 @@ GtkWidget *gtk_usergroup_new()
 	GtkUserGroup *usergroup;
 
 	usergroup= g_object_new (GTK_TYPE_USERGROUP, NULL);
-	gtk_window_set_title (GTK_WINDOW (usergroup), "Users & Groups Selection");
-	gtk_dialog_set_has_separator (GTK_DIALOG(usergroup), FALSE);
+	gtk_window_set_title (GTK_WINDOW (usergroup), _("Users & Groups Selection"));
+	//gtk_dialog_set_has_separator (GTK_DIALOG(usergroup), FALSE);
 
 	return GTK_WIDGET(usergroup);
 }
 
 
-//
-void gtk_usergroup_set_validate_user(GtkUserGroup *gtk_usergroup, GtkWidget *widget,
-	void *function)
-{
-	gtk_usergroup->validate_user_widget   = widget;
-	gtk_usergroup->validate_user_function = function;
-}
-
-
-//
-void gtk_usergroup_set_validate_group(GtkUserGroup *gtk_usergroup, GtkWidget *widget,
-	void *function)
-{
-	gtk_usergroup->validate_group_widget   = widget;
-	gtk_usergroup->validate_group_function = function;
-}
-
-
-//
-void gtk_usergroup_hide_users(GtkUserGroup *gtk_usergroup)
-{
-	gtk_widget_hide(gtk_usergroup->view_users);
-}
-
-
 /***** CALLBACKS *****/
 
+//When validate selection
+static void gtk_usergroup_on_validate(GtkWidget *button, gpointer *data)
+{
+	GtkUserGroup *gtk_usergroup;
+	void (*function) (GtkWidget *, gpointer *);
+	GtkTreeSelection *selection;
+	GtkTreeIter iter;
+	
+	gtk_usergroup = GTK_USERGROUP(data);
+	gtk_widget_hide(GTK_WIDGET(gtk_usergroup));
+	
+	//User selected
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(gtk_usergroup->view_users));
+	if (gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection), 
+					&gtk_usergroup->store_users, &iter)) {
+	
+		gchar *uid, *user;
+	
+		gtk_tree_model_get(GTK_TREE_MODEL(gtk_usergroup->store_users), 
+					&iter, 0, &uid, 1, &user, -1);
+		
+		gtk_usergroup->uid       = atoi(uid);
+		gtk_usergroup->user_name = g_strdup(user);
+				
+		g_log(0, G_LOG_LEVEL_DEBUG, "GtkUserGroup : Emit validate-user Signal", 0);
+		g_signal_emit(gtk_usergroup, gtk_usergroup_signals[VALIDATE_USER], NULL);
+		return;
+	}
+	
+	//Group selected
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(gtk_usergroup->view_groups));
+	if (gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection), 
+					&gtk_usergroup->store_users, &iter)) {
+		
+		gchar *gid, *group;
+		
+		gtk_tree_model_get(GTK_TREE_MODEL(gtk_usergroup->store_groups), 
+					&iter, 0, &gid, 1, &group, -1);
+		
+		
+		gtk_usergroup->gid        = atoi(gid);
+		gtk_usergroup->group_name = g_strdup(group);
+				
+						
+		g_signal_emit(gtk_usergroup, gtk_usergroup_signals[VALIDATE_GROUP], 0);
+		return;
+	}
+	
+}
+
 //
 static void gtk_usergroup_on_select_user(GtkTreeView *tree_view, gpointer user_data)
 {
@@ -244,56 +325,7 @@ static void gtk_usergroup_on_select_group(GtkTreeView *tree_view, gpointer user_
 }
 
 
-//
-static void gtk_usergroup_on_validate(GtkWidget *button, gpointer *data)
-{
-	GtkUserGroup *gtk_usergroup;
-	void (*function) (GtkWidget *, gpointer *);
-	GtkTreeSelection *selection;
-	GtkTreeIter iter;
-	
-	
-	gtk_usergroup = GTK_USERGROUP(data);
-	gtk_widget_hide(GTK_WIDGET(gtk_usergroup));
-	
-	
-	//User selected
-	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(gtk_usergroup->view_users));
-	if (gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection), 
-					&gtk_usergroup->list_users, &iter)) {
-	
-		gchar *uid, *user;
-	
-		gtk_tree_model_get(GTK_TREE_MODEL(gtk_usergroup->list_users), &iter, 0, &uid, 1, &user, -1);
-		
-		function = gtk_usergroup->validate_user_function;
-		if (function == NULL)
-			return;
-		
-		function(gtk_usergroup->validate_user_widget, atoi(uid));		
-		return;
-	}
-	
-	//Group selected
-	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(gtk_usergroup->view_groups));
-	if (gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection), 
-					&gtk_usergroup->list_users, &iter)) {
-		
-		gchar *gid, *group;
-		
-		gtk_tree_model_get(GTK_TREE_MODEL(gtk_usergroup->list_groups), 
-					&iter, 0, &gid, 1, &group, -1);
-		//g_print("GtkUserGroup : Selected group : %u - %s\n", atoi(gid), group);
-		
-		function = gtk_usergroup->validate_group_function;
-		if (function == NULL)
-			return;
-		
-		function(gtk_usergroup->validate_group_widget, atoi(gid));		
-		return;
-	}
-	
-}
+
 
 
 //
@@ -327,18 +359,12 @@ static void gtk_usergroup_on_user_clicked(GtkCellRenderer *cell, gchar *path_str
 /***** CORE DATA MANUPULATION FUNCTIONS *****/
 
 //
-static GtkWidget *_create_list_view(GtkListStore *list, gchar *text_header)
+static void init_list_view(GtkTreeView *view, gchar *text_header)
 {
-	GtkWidget *view;
+	//GtkWidget *view;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer   *cell_renderer;
 	
-
-	view = gtk_tree_view_new();
-	gtk_tree_view_set_model(GTK_TREE_VIEW(view), GTK_TREE_MODEL(list));
-	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), TRUE);
-	gtk_widget_set_size_request(view, 200, 300);
-	
 	column = gtk_tree_view_column_new();
 	gtk_tree_view_column_set_title(column, "id");
 	gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
@@ -355,14 +381,11 @@ static GtkWidget *_create_list_view(GtkListStore *list, gchar *text_header)
 	gtk_tree_view_column_add_attribute(column, cell_renderer, 
 		"text", 1);
 		
-	//gtk_container_add(GTK_CONTAINER(scrolled), view);
-	
-	return view;
 }
 	
 
 //
-static void _fill_users_list(GtkListStore *list_users)
+static void fill_users_list(GtkListStore *list_users)
 {
 	struct passwd *passwd;
 	GtkTreeIter  iter;
@@ -390,7 +413,7 @@ static void _fill_users_list(GtkListStore *list_users)
 
 
 //
-static void _fill_groups_list(GtkListStore *list_groups)
+static void fill_groups_list(GtkListStore *list_groups)
 {
 	struct group *group;
 	GtkTreeIter   iter;
