@@ -24,8 +24,9 @@
 
 #include "config.h"
 #include "gtk/gtk.h"
-
+#include "glade/glade.h"
 #include "gtkacl.h"
+#include "gtkacl-callbacks.h"
 
 
 //#include "bonobo_gtkacl.h"
@@ -37,17 +38,9 @@ void on_destroy(GtkWidget widget, gpointer data);
 //
 int main(gint argc, gchar **argv)
 {
-	GtkWidget *win_main;
-	GtkWidget *win_acl;
-	GtkWidget *vbox_main;
-	GList     *list_child;
-	
+	gtk_set_locale();
 	gtk_init(&argc, &argv);
 	
-	setlocale (LC_ALL, "");
-	bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
-	textdomain (PACKAGE);
-	
 	/***** Check parameters *****/
 	
 	if (argc < 2) 
@@ -56,22 +49,18 @@ int main(gint argc, gchar **argv)
 		exit(0);
 	}		
 	
+	if (g_strcasecmp(argv[1], "--help") == 0)
+	{
+		g_print(_("Usage : gtk-acl <filename>\n"));
+		exit(0);
+	}
+	
 	if (g_strcasecmp(argv[1], "--version") == 0)
 	{
 		g_print("%s version %s\n", PACKAGE, VERSION);
 		exit(0);
 	}
 
-	/*int fd;
-	fd = open(argv[1], O_RDONLY);
-	if (fd == -1)
-	{
-		close(fd);
-		g_print("Wrong path!\n");
-		return 1;
-	}
-	close(fd);*/
-	
 	struct stat s_stat;
 		
 	if (stat(argv[1], &s_stat) == -1)
@@ -80,28 +69,21 @@ int main(gint argc, gchar **argv)
 		return 1;
 	}
 	
-	/*create_bonobo_container(argc, argv);
-	exit(0);*/
-	
 	/***** *****/
+
+	GtkWidget *gtk_acl;
 	
-	win_main = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	gtk_window_set_title(GTK_WINDOW(win_main), _("ACL Management"));
-	g_signal_connect(win_main, "destroy", on_destroy, NULL);
-	gtk_widget_show(win_main);
-	
-	win_acl = gtk_acl_new();
-	gtk_acl_set_advanced_mode(GTK_ACL(win_acl), FALSE);
-	gtk_container_add(GTK_CONTAINER(win_main), win_acl);
-	gtk_widget_show(win_acl);
+	gtk_acl = gtk_acl_new();
+	g_signal_connect(G_OBJECT(gtk_acl), "destroy", G_CALLBACK(on_destroy), NULL);
+	gtk_widget_show(GTK_WIDGET(gtk_acl));
 	
 	
-	gtk_acl_load_file(GTK_ACL(win_acl), argv[1], TYPE_ACCESS);
-	//gtk_acl_load_file(GTK_ACL(win_acl), argv[1], TYPE_DEFAULT);
+	gtk_acl_load_file(GTK_ACL(gtk_acl), argv[1]);
 	
 	gtk_main();
+
+	return 0;
 	
-	return (0);
 }
 
 
